@@ -22,7 +22,7 @@ from jsonutils.functions.parsers import (
     parse_float,
     url_validator,
 )
-from jsonutils.query import QuerySet
+from jsonutils.query import ParentList, QuerySet
 from jsonutils.utils.dict import UUIDdict
 from jsonutils.utils.retry import retry_function
 
@@ -37,7 +37,7 @@ class JSONPath:
         obj = super().__new__(cls)
         obj._string = s  # pretty path
         obj._path = ""  # python json path
-        obj._keys = [] # list of dict keys
+        obj._keys = []  # list of dict keys
         return obj
 
     @property
@@ -184,6 +184,19 @@ class JSONNode:
             parent = parent.parent
         return path
 
+    @property
+    def parent_list(self):
+        pl = ParentList()
+
+        parent = self
+        last = parent
+        while parent is not None:
+            last = parent
+            parent = parent.parent
+            pl.append(last)
+
+        return pl[1:]
+
     @property
     def root(self):
         """Get root object from current node object"""
@@ -279,6 +292,10 @@ class JSONNode:
 
         return _path(self, other)
 
+    def notpath_action(self, other):
+        from jsonutils.functions.actions import _notpath
+
+        return _notpath(self, other)
 
 
 class JSONCompose(JSONNode):
@@ -298,6 +315,10 @@ class JSONCompose(JSONNode):
         self._child_objects = UUIDdict()
         self._assign_children()
 
+    @property
+    def _data(self):
+        return super().json_decode
+
     def _assign_children(self):
         """Any JSON object can be a child for a given compose object"""
         if isinstance(self, JSONDict):
@@ -349,7 +370,14 @@ class JSONCompose(JSONNode):
                 )
         return queryset
 
-    def get(self, recursive_=None, include_parent_=None, throw_exceptions_=None, **q):
+    def get(
+        self,
+        recursive_=None,
+        include_parent_=None,
+        throw_exceptions_=None,
+        native_types_=None,
+        **q,
+    ):
 
         # ---- DYNAMIC CONFIG ----
         if recursive_ is None:
@@ -358,6 +386,8 @@ class JSONCompose(JSONNode):
             include_parent_ = config.include_parents
         if throw_exceptions_ is None:
             throw_exceptions_ = config.query_exceptions
+        if native_types_ is None:
+            native_types_ = config.native_types
         # ------------------------
         query = self.query(
             recursive_=recursive_,
@@ -377,7 +407,11 @@ class JSONCompose(JSONNode):
             else:
                 return query.first()
         else:
-            return query.first()
+            result = query.first(native_types_=False)
+            if native_types_:
+                return result._data
+            else:
+                return result
 
     def annotate(self, **kwargs):
         """
@@ -691,7 +725,7 @@ class JSONList(list, JSONCompose):
 class JSONStr(str, JSONSingleton):
     def __new__(cls, string):
         obj = super().__new__(cls, string)
-        obj._data = string
+        obj._data = str(string)
         return obj
 
     # converters
@@ -898,7 +932,7 @@ class JSONStr(str, JSONSingleton):
 class JSONFloat(float, JSONSingleton):
     def __new__(cls, fl):
         obj = super().__new__(cls, fl)
-        obj._data = fl
+        obj._data = float(fl)
         return obj
 
     def __hash__(self):
@@ -938,7 +972,7 @@ class JSONFloat(float, JSONSingleton):
 class JSONInt(int, JSONSingleton):
     def __new__(cls, i):
         obj = super().__new__(cls, i)
-        obj._data = i
+        obj._data = int(i)
         return obj
 
     def __hash__(self):
@@ -996,6 +1030,12 @@ class JSONBool(JSONSingleton):
         except Exception:
             return False
 
+    def __ne__(self, other):
+        try:
+            return self._data != parse_bool(other)
+        except Exception:
+            return False
+
     def __gt__(self, other):
         return False
 
@@ -1029,6 +1069,12 @@ class JSONNull(JSONSingleton):
         except Exception:
             return False
 
+    def __ne__(self, other):
+        try:
+            return self._data != other
+        except Exception:
+            return False
+
     def __gt__(self, other):
         return False
 
