@@ -19,15 +19,80 @@ def _parse_query(node, include_parent_, **q):
     Query q must be structured as follows:
         <key>__<modificator>__<query>
     """
-    # TODO if a query contains two different keys, take into account the dict
     from jsonutils.base import JSONNode
 
+    # grab a list of node actions, without the action suffix
+    node_actions = [
+        i.replace("_action", "") for i in dir(JSONNode) if i.endswith("action")
+    ]
+
     class UniqueList(list):
         """A list but appending only unique values"""
 
         def append(self, object):
             return super().append(object) if object not in self else None
 
+    def make_actions(obj, target_actions, query_value):
+        actions_count = len(target_actions)
+        for idx, action in enumerate(target_actions):
+
+            # ---- MODIFICATORS ----
+            # modify obj before apply actions
+            if action == "parent":
+                obj = obj.parent
+                if obj is None:
+                    return False
+                if idx == actions_count - 1:
+                    action = "exact"  # if parent is last action, take exact as the default one
+                else:
+                    continue  # continue to next action
+            elif action == "parents": # multiparents modificator
+                parents = obj.parent_list
+                if not parents:
+                    return False
+                if "parents" in target_actions[idx + 1 :]:
+                    raise JSONQueryException("Lookup parents can only be included once")
+
+                results = (
+                    make_actions(i, target_actions[idx + 1 :], query_value)
+                    for i in parents
+                )
+                if any(results):
+                    return True  # no more actions, continue with next query
+                else:
+                    return False
+            elif match := re.fullmatch(r"c_(\w+)", action):  # child modificator
+                try:
+                    obj = obj.__getitem__(match.group(1))
+                except Exception:
+                    return False
+                if idx == actions_count - 1:
+                    action = "exact"  # if child is last action, take exact as the default one
+                else:
+                    continue  # continue to next action
+            elif action.isdigit():
+                if not isinstance(obj, list):
+                    return False
+                try:
+                    obj = obj[int(action)]
+                except IndexError:
+                    return False
+                if idx == actions_count - 1:
+                    action = "exact"  # if digit is last action, take exact as the default one
+                else:
+                    continue  # continue to next action
+            # ---- MATCH ----
+            # all comparisons have child object to the left, and the underlying algorithm is contained in the magic methods of the JSON objects
+            # no errors will be thrown, if types are not compatible, just returns False
+            # node actions can't interfer with modificators
+            if action in node_actions:  # call corresponding node method
+                result = getattr(obj, action + "_action")(query_value)
+                if not result:
+                    return False
+            else:
+                raise JSONQueryException(f"Bad query: {action}")
+        return True
+
     target_keys = UniqueList()
 
     for query_key, query_value in q.items():
@@ -66,63 +131,16 @@ def _parse_query(node, include_parent_, **q):
             # so we prepend __parent__c_<target_key> in the target_actions list
             target_actions = ["parent", f"c_{target_key}"] + target_actions
             target_key = target_keys[0]
-            include_parent_ = True
 
         # first of all, if target key of query argument does not match child's key, we won't append it to querylist
         if target_key != node._key:
             return False, None
 
-        # grab a list of node actions, without the action suffix
-        node_actions = [
-            i.replace("_action", "") for i in dir(JSONNode) if i.endswith("action")
-        ]
         obj = node
 
-        actions_count = len(target_actions)
-
-        for idx, action in enumerate(target_actions):
-
-            # ---- MODIFICATORS ----
-            # modify obj before apply actions
-            if action == "parent":
-                obj = obj.parent
-                if obj is None:
-                    return False, None
-                if idx == actions_count - 1:
-                    action = "exact"  # if parent is last action, take exact as the default one
-                else:
-                    continue  # continue to next action
-            elif match := re.fullmatch(r"c_(\w+)", action):  # child modificator
-                try:
-                    obj = obj.__getitem__(match.group(1))
-                except Exception:
-                    return False, None
-                if idx == actions_count - 1:
-                    action = "exact"  # if child is last action, take exact as the default one
-                else:
-                    continue  # continue to next action
-            elif action.isdigit():
-                if not isinstance(obj, list):
-                    return False, None
-                try:
-                    obj = obj[int(action)]
-                except IndexError:
-                    return False, None
-                if idx == actions_count - 1:
-                    action = "exact"  # if digit is last action, take exact as the default one
-                else:
-                    continue  # continue to next action
-            # ---- MATCH ----
-            # all comparisons have child object to the left, and the underlying algorithm is contained in the magic methods of the JSON objects
-            # no errors will be thrown, if types are not compatible, just returns False
-            # node actions can't interfer with modificators
-            if action in node_actions:  # call corresponding node method
-                result = getattr(obj, action + "_action")(query_value)
-                if not result:
-                    return False, None
-            else:
-                raise JSONQueryException(f"Bad query: {action}")
-
+        result = make_actions(obj, target_actions, query_value)
+        if result is False:
+            return False, None
     return (True, node.parent if include_parent_ else node)
 
 
