@@ -2,7 +2,10 @@ import re
 from datetime import date, datetime
 from typing import Union
 
+import jsonutils.base as base
+import jsonutils.config as config
 from jsonutils.exceptions import JSONQueryException
+import jsonutils.functions.parsers as parsers
 
 
 class SingleQuery:
@@ -62,9 +65,8 @@ class SingleQuery:
         Check this single query against a target node object.
         node argument must be an instance of JSONNode.
         """
-        from jsonutils.base import JSONNode
 
-        if not isinstance(node, JSONNode):
+        if not isinstance(node, base.JSONNode):
             raise JSONQueryException(
                 f"child argument must be JSONNode type, not {type(node)}"
             )
@@ -74,7 +76,7 @@ class SingleQuery:
 
         # grab a list of node actions, without the action suffix
         node_actions = [
-            i.replace("_action", "") for i in dir(JSONNode) if i.endswith("action")
+            i.replace("_action", "") for i in dir(base.JSONNode) if i.endswith("action")
         ]
         obj = node
 
@@ -180,6 +182,10 @@ class Q:
         return obj
 
 
+class ParentList(list):
+    pass
+
+
 class QuerySet(list):
     """
     This is a queryset object.
@@ -192,11 +198,27 @@ class QuerySet(list):
         super().__init__(*args)
         self._root = None
 
-    def first(self):
-        return self.__getitem__(0) if self.__len__() > 0 else None
+    def first(self, native_types_=None):
+        # ---- DYNAMIC CONFIG ----
+        if native_types_ is None:
+            native_types_ = config.native_types
+        # ------------------------
+        result = self.__getitem__(0)
+        if not native_types_:
+            return result if self.__len__() > 0 else None
+        else:  # if native_types_ is selected
+            return result._data if self.__len__() > 0 else None
 
-    def last(self):
-        return self.__getitem__(-1) if self.__len__() > 0 else None
+    def last(self, native_types_=None):
+        # ---- DYNAMIC CONFIG ----
+        if native_types_ is None:
+            native_types_ = config.native_types
+        # ------------------------
+        result = self.__getitem__(-1)
+        if not native_types_:
+            return result if self.__len__() > 0 else None
+        else:  # if native_types_ is selected
+            return result._data if self.__len__() > 0 else None
 
     def exists(self):
         return True if self.__len__() > 0 else False
@@ -226,7 +248,6 @@ class QuerySet(list):
         """
         Returns unique values in a querylist
         """
-        from jsonutils.base import JSONSingleton
 
         # TODO add test for this
         # TODO dict option for counting unique values
@@ -246,6 +267,19 @@ class QuerySet(list):
                 output.append(item)
         return output
 
+    # ---- GROUP OPERATIONS ----
+    def sum(self):
+        """Sum numbers on queryset"""
+
+        if not self.exists():
+            return
+
+        result = 0
+        for item in self:
+            number = parsers.parse_float(item, fail_silently=True) or 0
+            result += number
+        return result
+
     def __repr__(self):
         return "<QuerySet " + super().__repr__() + ">"
 
