@@ -1,8 +1,8 @@
 import json
-from jsonutils.exceptions import JSONQueryException, JSONQueryMultipleValues
 import unittest
 from datetime import datetime
 
+import jsonutils as js
 import pytz
 from jsonutils.base import (
     JSONBool,
@@ -19,11 +19,16 @@ from jsonutils.base import (
     JSONStr,
 )
 from jsonutils.encoders import JSONObjectEncoder
+from jsonutils.exceptions import JSONQueryException, JSONQueryMultipleValues
 from jsonutils.query import All, QuerySet, SingleQuery
 
 
 class JsonTest(unittest.TestCase):
     def setUp(self):
+
+        js.config.native_types = False
+        js.config.query_exceptions = True
+
         self.test1 = JSONObject(
             [
                 {"Float": 2.3, "Int": 1, "Str": "string"},
@@ -338,12 +343,27 @@ class JsonTest(unittest.TestCase):
 
     def test_queries(self):
 
+        self.assertEqual(JSONObject(dict(A=True)).query(A__isnull=True), [])
+        self.assertEqual(JSONObject(dict(A=True)).query(A__isnull=False), [True])
+        self.assertEqual(JSONObject(dict(A=False)).query(A__isnull=True), [])
+        self.assertEqual(JSONObject(dict(A=False)).query(A__isnull=False), [False])
         self.assertEqual(self.test3.query(Bool="true"), [JSONBool(True)])
         self.assertEqual(self.test3.query(Bool__type=str), [])
+        self.assertEqual(self.test3.query(Bool__type="str"), [])
         self.assertEqual(self.test3.query(Bool__type=bool), [JSONBool(True)])
+        self.assertEqual(self.test3.query(Bool__type="bool"), [JSONBool(True)])
         self.assertEqual(self.test4.query(None__type=None), [JSONNull(None)])
+        self.assertEqual(self.test4.query(None__type="None"), [JSONNull(None)])
+        self.assertEqual(self.test5.query(Str__type="str"), ["string1", "string2"])
+        self.assertEqual(self.test5.query(Str__type="None"), [])
         self.assertEqual(self.test5.query(Str__length=7), ["string1", "string2"])
         self.assertEqual(self.test5.query(Str__length=0), [])
+        self.assertEqual(
+            self.test5.query(Str__type="singleton"), ["string1", "string2"]
+        )
+        self.assertEqual(
+            self.test5.query(Datetime__type=datetime), self.test5.query(Datetime=All)
+        )
         self.assertRaises(JSONQueryException, lambda: self.test5.query(Str__length="1"))
         self.assertEqual(
             self.test3.query(List__contains=True), [[JSONBool(True), JSONBool(False)]]
@@ -456,7 +476,9 @@ class JsonTest(unittest.TestCase):
         )
 
         self.assertEqual(
-            self.test6.query(text__regex=r"(?:2|5)", pos__0__gte=2),
+            self.test6.query(
+                text__regex=r"(?:2|5)", pos__0__gte=2, include_parent_=True
+            ),
             [
                 {"text": "dummy text 2", "pos": [3, 2]},
                 {"text": "dummy text 5", "pos": [4, 1]},
@@ -464,7 +486,7 @@ class JsonTest(unittest.TestCase):
         )
 
         self.assertEqual(
-            test.query(data=1, date__gte=datetime(2021, 8, 1)),
+            test.query(data=1, date__gte=datetime(2021, 8, 1), include_parent_=True),
             [
                 {"data": 1, "date": "2021/08/01  12:00:00Z"},
                 {"data": 1, "date": "2021-08-02"},
@@ -486,7 +508,9 @@ class JsonTest(unittest.TestCase):
             ],
         )
         self.assertEqual(
-            test.annotate(date=None).query(data__c_data__contains=1, date__isnull=True),
+            test.annotate(date=None).query(
+                data__c_data__contains=1, date__isnull=True, include_parent_=True
+            ),
             [{"data": {"data": 1, "date": "2021-08-02"}, "date": None}],
         )
 
@@ -747,3 +771,41 @@ class JsonTest(unittest.TestCase):
         self.assertSetEqual(set(test.data._0._child_objects.values()), {30})
         self.assertFalse(test.query(name=All).exists())
         self.assertRaises(AttributeError, lambda: test.data._0.name)
+
+    def test_multiparent(self):
+        test = JSONObject(
+            {
+                "root": {
+                    "root_list": [0, {"child": {"A": 1}}, {"child2": {"A": 1}}],
+                    "root_dict": {"child": {"A": 1}, "key": [0, "key"]},
+                }
+            }
+        )
+
+        self.assertEqual(test.query(A__parents__c_key__1__contains="ey"), ["1"])
+        self.assertEqual(test.query(A__parents__c_key__1__contains="ey", A=1), ["1"])
+        self.assertFalse(
+            test.query(A__parents__c_key__1__contains="ey", A__gt=1).exists()
+        )
+        self.assertEqual(test.query(A__parents__c_key__1__contains="ey").count(), 1)
+        self.assertEqual(
+            test.query(A__parents__c_key__1__contains="ey").first().jsonpath,
+            "root/root_dict/child/A/",
+        )
+        self.assertEqual(
+            test.get(A__parents__0=0, A__parents__index=1, throw_exceptions_=True), 1
+        )
+        self.assertEqual(
+            test.get(
+                A__parents__0=0, A__parents__index=1, throw_exceptions_=True
+            ).jsonpath,
+            "root/root_list/1/child/A/",
+        )
+        self.assertIsNone(
+            test.get(A__parents__0=0, A__parents__index=0, throw_exceptions_=False)
+        )
+        self.assertRaisesRegex(
+            JSONQueryException,
+            "Lookup parents can only be included once",
+            lambda: test.get(A__parents__parents__0=0),
+        )
